#! /bin/sh
#
# PC Card Wizard Release 1.11 for BeOS R5 x86
# Copyright (C) ITO Takayuki <yuki@fsinet.or.jp>
# http://hp.vector.co.jp/authors/VA001240/archive/index-en.html
#

IDENT=/tmp/ident$$.txt
CIS=/tmp/cis$$.txt
CARDID=/tmp/cardid$$.txt
SYSTEM_DATABASE=$HOME/config/settings/kernel/drivers/pcmcia.default
USER_DATABASE=$HOME/config/settings/kernel/drivers/pcmcia

cd "`dirname \"$0\"`"		# for being called from Tracker

#
# finish
#

finish ()
{
	rm -f $IDENT $CARDID $CIS
	[ $1 -gt 0 ] && alert --stop "$MSG_ABORT" $BTN_OK > /dev/null
	exit $1
}

#
# set $PCMCIANETWORK
#

setpcmcianetwork ()
{
	PCMCIANETWORK=
	cd $HOME/config/boot
	if [ -f UserSetupEnvironment ]
	then
		. UserSetupEnvironment
		if [ "$PCMCIANETWORK" = true ]
		then
			return 0
		fi
	fi

	alert --warning "$MSG_SUPPRESS_NET" $BTN_NO $BTN_YES > /dev/null
	if [ $? -eq 1 ]
	then
		[ -f UserSetupEnvironment ] || cp UserSetupEnvironment.sample UserSetupEnvironment
		cat >> UserSetupEnvironment << END

# begin: added by PC Card Wizard
export PCMCIANETWORK=true
# end: added by PC Card Wizard

END
		alert --info "$MSG_NET_SUPPRESSED" $BTN_OK > /dev/null
	fi
}

recognize ()
{
#
# database update
#
cat > $CARDID << END
card { # $CARDNAME
	version $VERS1
END
[ -z "$MANFID" ] || \
cat >> $CARDID << END
	manfid $MANFID
END
cat >> $CARDID << END
	bind $DRIVER
}
END

alert --warning "$MSG_REGISTER_CARD

`cat $CARDID`" $BTN_QUIT $BTN_NEXT > /dev/null && finish 1
cat >> $USER_DATABASE << END

# begin: added by PC Card Wizard
`[ "$NE2000INCOMPAT" ] && echo "# WARNING: this card is unlikely to be NE2000 compatible!!"; cat $CARDID`
# end: added by PC Card Wizard

END
alert --info "$MSG_CARD_REGISTERED" $BTN_OK > /dev/null

#
# try to recognize
#

#alert --warning "$MSG_REFRESH_CARD" $BTN_QUIT $BTN_NEXT > /dev/null && finish 1
cardctl eject
sleep 1
cardctl insert
i=0
until [ "$RECOGNIZED" -o $i -ge 5 ]
do
	sleep 1
	case $FUNCID in
	2)
		if [ -e /dev/ports/serial$NEXTSERIAL ]
		then
			alert --info "$MSG_CARD_RECOGNIZED" $BTN_OK > /dev/null
			RECOGNIZED=1
		fi
		;;
	4)
		if [ -e /dev/disk/ide/ata/$NEXTATA -o \
		     -e /dev/disk/ide/atapi/$NEXTATA ]
		then
			alert --info "$MSG_CARD_RECOGNIZED" $BTN_OK > /dev/null
			RECOGNIZED=1
		fi
		;;
	6|16)
		if [ -e /dev/net/$DRIVER/0 ]
		then
			alert --info "$MSG_CARD_RECOGNIZED" $BTN_OK > /dev/null
			RECOGNIZED=1
		fi
		;;
	esac
	i=$(($i + 1))
done
if [ -z "$RECOGNIZED" ]
then
	alert --stop "$MSG_RECOGNITION_FAILED

$MSG_IRQ_PROBLEM

$MSG_ATA_PROBLEM" $BTN_QUIT > /dev/null
	finish 1
fi
}

#
# get last serial port
#

getlastserial()
{
	if [ -e /dev/ports/serial1 ]
	then
		LASTSERIAL=$(expr "`echo /dev/ports/serial*`" : '.*serial\([0-9]*\)$')
	else
		LASTSERIAL=0
	fi
}

#
# get last ATA bus
#

getlastata()
{
	[ -d /dev/disk/ide/ata   ] && LASTATA=$(expr "`echo /dev/disk/ide/ata/*`" : '.*/\([0-9]*\)$')
	[ -d /dev/disk/ide/atapi ] && LASTATAPI=$(expr "`echo /dev/disk/ide/atapi/*`" : '.*/\([0-9]*\)$')
	[ "$LASTATAPI" ] && [ "$LASTATAPI" -gt $LASTATA ] && LASTATA=$LASTATAPI
}

#
# main
#

case `uname -r` in
4.6|5.0)
	;;
3.0|3.1|3.2|4.0|4.1|4.5)
	alert --stop "PC Card Wizard does not work on this version of BeOS." Quit > /dev/null
	exit 1
	;;
*)
	alert --warning "PC Card Wizard is not tested on this version of BeOS."
	exit 1
	;;
esac

#
# language choice
#

if [ "$LANG" = C -o "$LANG" = POSIX ]
then
	LANG=en_US
elif [ "x$LANG" != x ]
then
	LANG=`echo $LANG | cut -c 1-5`
elif [ -f wr_WR.msg ]
then
	. wr_WR.msg
	group=0
	while [ "x$LANG" = x ]
	do
		alert --empty "$MSG_CHOOSE_LANG

$MSG_LANG_CAUTION" ${BTN_LANG0[$group]} ${BTN_LANG1[$group]} Others > /dev/null
		case $? in
		0)
			LANG=${LANG0[$group]}
			;;
		1)
			LANG=${LANG1[$group]}
			;;
		2)
			group=$(($group + 1))
			[ $group = $GRPS ] && group=0
			;;
		esac
	done
#else
#	LANG=en_US
fi

if [ ! -f $LANG.msg ]
then
	for f in en_US.msg *.msg none
	do
		[ $f != wr_WR.msg -a -f $f ] && break
	done
	if [ $f = none ]
	then
		alert --stop "Message catalog files not found."
		exit 1
	fi
	l=`basename $f .msg`
	[ "x$LANG" != x ] && alert --info "Specified language $LANG not found. Falling back to $l."
	LANG=$l
fi

. $LANG.msg

#
# opening
#

alert --info "$MSG_WELCOME" $BTN_QUIT $BTN_NEXT > /dev/null && finish 1

#
# count sockets
#

if [ ! -e /dev/bus/pcmcia/sock/0 ]
then
	alert --stop "$MSG_NO_SOCKET" $BTN_QUIT > /dev/null
	finish 1
fi

pushd /dev/bus/pcmcia/sock > /dev/null
SOCKETS=`echo *`
popd > /dev/null

#
# card removal
#

cardctl eject
sleep 1

getlastserial
NEXTSERIAL=$(($LASTSERIAL + 1))
getlastata
if [ $LASTATA -lt 2 ]
then
	NEXTATA=2
else
	NEXTATA=$(($LASTATA + 1))
fi

#
# card insertion
#

until cardctl status | grep -q "card present"
do
	alert --warning $STYLE "$MSG_INSERT" $BTN_QUIT $BTN_NEXT > /dev/null && finish 1
	cardctl insert 2> /dev/null
	sleep 1
done

#
# card detection
#

for SOCKET in $SOCKETS
do
	if cardctl status $SOCKET | grep -q "card present"
	then
		break
	fi
done

#
# gather card info
#

cardctl ident $SOCKET > $IDENT
VERS1=`grep 'product info:' $IDENT | sed -e 's/^[^"]*\(.*\)[^"]*$/\1/' -e 's/", "/" "/g'`
VENDOR=`expr "$VERS1" : '\("[^"]*"\)'`
PRODUCT=`expr "$VERS1" : '"[^"]*" \("[^"]*"\)'`
MANFID=`grep manfid: $IDENT | sed 's/.*\(0x....\), \(0x....\)/\1 \2/'`
CARDNAME=`echo $VENDOR $PRODUCT | tr -d \"`

#
# check the card
#

FUNCID=`grep function: $IDENT | awk '{print $2}'`
[ -z "$FUNCID" ] && dump_cis | grep -q 'io 0x01.0-0x01.7, 0x03.6-0x03.7' && FUNCID=4
[ "$FUNCID" = 6 ] && expr "$MANFID" : '.*0x0589$' > /dev/null && FUNCID=16

case $FUNCID in
2)
	DRIVER=zz
	if [ -e /dev/ports/serial$NEXTSERIAL ]
	then
		alert --info "$MSG_KNOWN_SERIAL_INSERTED" $BTN_OK > /dev/null
		RECOGNIZED=1
	elif grep -v '^#' $SYSTEM_DATABASE | grep -q "$VERS1" || \
	     grep -v '^#'   $USER_DATABASE | grep -q "$VERS1" 2> /dev/null
	then
		alert --stop "$MSG_REGISTERED_NOT_RECOGNIZED

$MSG_IRQ_PROBLEM" $BTN_QUIT > /dev/null
		finish 1
	else
		alert --info "$MSG_NEW_SERIAL_INSERTED" $BTN_OK > /dev/null
	fi
	;;
4)
	if [ ! -f /system/add-ons/kernel/busses/ide/pcmcia ]
	then
		alert --stop "$MSG_NO_ATA_DRIVER" $BTN_QUIT > /dev/null
		finish 1
	fi

	DRIVER=ide_cs
	if [ -e /dev/disk/ide/ata/$NEXTATA -o -e /dev/disk/ide/atapi/$NEXTATA ]
	then
		alert --info "$MSG_KNOWN_ATA_INSERTED" $BTN_OK > /dev/null
		RECOGNIZED=1
	elif grep -v '^#' $SYSTEM_DATABASE | grep -q "$VERS1" || \
	     grep -v '^#'   $USER_DATABASE | grep -q "$VERS1" 2> /dev/null
	then
		alert --stop "$MSG_REGISTERED_NOT_RECOGNIZED

$MSG_IRQ_PROBLEM

$MSG_ATA_PROBLEM" $BTN_QUIT > /dev/null
		finish 1
	else
		alert --info "$MSG_NEW_ATA_INSERTED" $BTN_OK > /dev/null
	fi
	;;
6)
	DRIVER=etherpcmcia
	if [ -e /dev/net/etherpcmcia/0 ]
	then
		alert --info "$MSG_KNOWN_NETWORK_INSERTED" $BTN_OK > /dev/null
		RECOGNIZED=1
	elif grep -v '^#' $SYSTEM_DATABASE | grep -q "$VERS1" || \
	     grep -v '^#'   $USER_DATABASE | grep -q "$VERS1" 2> /dev/null
	then
		alert --stop "$MSG_REGISTERED_NOT_RECOGNIZED

$MSG_IRQ_PROBLEM" $BTN_QUIT > /dev/null
		finish 1
	elif dump_cis | grep -q 'io 0x..00-0x..1f'
	then
		alert --info "$MSG_NE2000_INSERTED" $BTN_OK > /dev/null
	else
		alert --stop "$MSG_NON_NE2000_INSERTED" $BTN_QUIT > /dev/null
		finish 1
	fi
	;;
16)
	if [ ! -f /system/add-ons/kernel/drivers/bin/tcm589 -o \
	     ! -f /system/add-ons/net_server/tcm589 ]
	then
		alert --stop "$MSG_NO_3C589_DRIVER" $BTN_QUIT > /dev/null
		finish 1
	fi

	DRIVER=tcm589
	if [ -e /dev/net/tcm589/0 ]
	then
		alert --info "$MSG_KNOWN_NETWORK_INSERTED" $BTN_OK > /dev/null
		RECOGNIZED=1
	elif grep -v '^#' $SYSTEM_DATABASE | grep -q "$VERS1" || \
	     grep -v '^#'   $USER_DATABASE | grep -q "$VERS1" 2> /dev/null
	then
		alert --stop "$MSG_REGISTERED_NOT_RECOGNIZED

$MSG_IRQ_PROBLEM" $BTN_QUIT > /dev/null
		finish 1
	else
		alert --info "$MSG_NEW_3C589_INSERTED" $BTN_OK > /dev/null
	fi
	;;
*)
	alert --stop "$MSG_UNKNOWN_CARD" $BTN_QUIT > /dev/null
	finish 1
	;;
esac

if [ -z "$RECOGNIZED" ]
then
	recognize
fi

#
# finish
#

case $FUNCID in
2)
	alert --empty "$MSG_COMPLETE

$MSG_GOTO_DUN" $BTN_END $BTN_NEXT > /dev/null && finish 0
	DialUpNetworking &
	;;
4)
	alert --empty "$MSG_COMPLETE

$MSG_GOTO_DRIVESETUP" $BTN_END $BTN_NEXT > /dev/null && finish 0
	DriveSetup &
	;;
6)
	setpcmcianetwork
	alert --empty "$MSG_COMPLETE

$MSG_GOTO_NET" $BTN_END $BTN_NEXT > /dev/null && finish 0
	Network &
	;;
16)
	setpcmcianetwork
	alert --empty "$MSG_COMPLETE

$MSG_GOTO_NET_3C589" $BTN_END $BTN_NEXT > /dev/null && finish 0
	Network &
	;;
esac

finish 0
